/******************************************************************
sock_serv.C
HummingBird Project - Deborah A. Frincke, PhD.
Spring 1998
Coded by: Jamie Marconi
Description: This class is used for server ports.
*******************************************************************/
/*
 Copyright (C) 1997-1998 The HummingBird Project - hummer@cs.uidaho.edu

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "sock_serv.h"

/********
Function: sock_serv::sock_serv
Purpose:  Basic constructor.  Init. some varibles.
********/

sock_serv::sock_serv()
{
  asock = -1;
  service = NULL;

#ifdef KRB5
  auth_context = NULL;
#endif

}

/**********
Function: sock_serv::sock_serv
Purpose:  Basic constructor. Takes the port to bind to and and the number of
          connections listen can keep backlogged.
**********/

sock_serv::sock_serv(int pport, int listen_backlog = LISTEN_BACKLOG)
{
  service = NULL;
  asock = -1;

#ifdef KRB5
  auth_context = NULL;
#endif

  create();
  bind(pport);
  listen(listen_backlog);
}


/**********
Function: sock_serv::sock_serv
Purpose:  Basic constructor. Takes the port to bind to, the service name, and
          and the number of connections listen can keep backlogged.
**********/

sock_serv::sock_serv(int pport, char *pservice, int listen_backlog = LISTEN_BACKLOG)
{
  service = NULL;
  asock = -1;

#ifdef KRB5
  auth_context = NULL;
#endif

  create();
  bind(pport, pservice);
  listen(listen_backlog);
}


/**********
Function: sock_serv::sock_serv
Purpose:  Basic constructor. Takes the service name to bind to and and the 
          number of connections listen can keep backlogged.
**********/

sock_serv::sock_serv(char* pservice, int listen_backlog = LISTEN_BACKLOG)
{
  service = NULL;
  asock = -1;

#ifdef KRB5
  auth_context = NULL;
#endif

  create();
  bind(pservice);
  listen(listen_backlog);
}

/********
Function: sock_serv::~sock_serv
Purpose:  Basic destructor.
********/

sock_serv::~sock_serv()
{
#ifdef KRB5
  if (auth_context)
    krb5_auth_con_free(context, auth_context);

  auth_context = NULL;
#endif

  sock = -1;
  asock = -1;
}

/********
Function: sock_serv::bind
Purpose:  Calls bind and binds the socket to the port assocated with the 
          pservice returns 0 on error, 1 on success.
*********/

int sock_serv::bind(char *pservice)
{
  struct servent *serv;
  
  if (!pservice) {
#ifdef VERBOSE
    Print_Error("sock_serv::bind", "empty service name without port");
#endif
    return 0;
  } 

  if ((serv = getservbyname(pservice, "tcp")) ==  NULL) {
#ifdef VERBOSE
    Print_Error("sock_serv::bind", "service unknown: %s/tcp", pservice);
#endif
    return 0;
  }
  
  return bind(ntohs(serv->s_port), pservice);
} 

/********
Function: sock_serv::bind
Purpose:  Calls bind and binds the socket to the port assocated with the 
          pservice returns 0 on error, 1 on success.
*********/

int sock_serv::bind(int pport, char *pservice)
{
  service = strdup(pservice);
  
  return bind(pport);
}


/********
Function: sock_serv::bind
Purpose:  Calls bind and binds the socket to the port 
          returns 0 on error, 1 on success.
*********/

int sock_serv::bind(int pport)
{
  // port = pport;

  struct sockaddr_in s_addr_in;
  
  // Setup the port.
  s_addr_in.sin_family = AF_INET;
  s_addr_in.sin_addr.s_addr = INADDR_ANY;
  s_addr_in.sin_port = htons(pport);

  // Bind the port to the socket
  if(::bind(sock, (struct sockaddr *)&s_addr_in, sin_len) == -1 ) {
#ifdef VERBOSE
    Print_Error("sock_serv::bind", "Cannot bind to port: %s", strerror(errno));
#endif
    return 0;
  }
  
  /* Set socket options.  We try to make the port reusable and have it
     close as fast as possible without waiting in unnecessary wait states
     on close. */
  int on = 1;
  setsockopt(sock, SOL_SOCKET, SO_REUSEADDR,(char *) &on, sizeof(on));
#ifdef SO_LINGER
  struct linger ling;
  ling.l_onoff = 1;
  ling.l_linger = 15;
  setsockopt(sock, SOL_SOCKET, SO_LINGER, (char *) &ling, sizeof(ling));
#endif /* SO_LINGER */

  return 1;
}

/*******
Function: sock_serv::close_a
Purpose:  Frees the files descriptor and closes the socket connection.
*******/ 
int sock_serv::close_a()
{
  if (::close(asock) == -1) 
    return 0;

  return 1;
}

/********
Function: sock_serv::listen
Purpose:  Calls the system function listen to make the port listen for 
connections.
********/

int sock_serv::listen(int listen_backlog = LISTEN_BACKLOG)
{	
  if (::listen(sock, listen_backlog) == -1) 
    return 0;
  else
    return 1;
}

/********
Function: sock_serv::accept
Purpose:  Calls the system function accpet to block until a connection to 
the port is experienced.
********/

int sock_serv::accept()
{
  if ((asock = ::accept(sock, (struct sockaddr *) rmt_sockaddr, &sin_len)) != -1) {

    if (getpeername(asock, (struct sockaddr *) rmt_sockaddr, &sin_len) == -1) {
#ifdef VERBOSE
      Print_Error("sock_serv::accept", "getpeername error %s", strerror(errno));
#endif
      return 0;
    }
    else {
      return 1;
    }
  }
  else {
#ifdef VERBOSE
    Print_Error("sock_serv::accept", "accept: %s", strerror(errno));
#endif

    return 0;
  }
}

/********
Function: sock_serv::get_asock
Purpose:  Returns the new file descriptor of the accepted socket.
********/

int sock_serv::get_asock(void)
{
  return asock;
}

/********
Function: sock_serv::get_rmt_addr
Purpose:  Gets the IP of the connecting host.
********/

void sock_serv::get_rmt_addr(char *buf)
{
  strcpy(buf,inet_ntoa(rmt_sockaddr->sin_addr));
} 


/********
Function: sock_serv::make_sock_cli
Purpose:  Create a sock_cli object from the accepted socket
          Auth info is not copied to client object
*********/
sock_cli *sock_serv::make_sock_cli()
{
  sock_cli *nc;
  nc = new sock_cli(asock, service);
  
  return nc;
}
